#!/usr/bin/perl -w
# rotate_antenna_model.pl       O.Charade           2009 Sep. 17.

use strict;
use Readonly;
use List::MoreUtils qw ( any );
use Data::Dumper;

Readonly my $ANTMOD_FILE    =>  '/home/parcinsu/gg/tables/antmod.dat';
Readonly my @STEADY_LINES   => ('METH / BY / # / DATE',
                                'DAZI',
                                'ZEN1 / ZEN2 / DZEN',
                                'COMMENT',
                                );

Readonly my $USAGE          =>  << 'END_USAGE';
usage: rotate_antenna_model.pl ANTENNA_TYPE

scans ~/gg/tables/antmod.dat looking for ANTENNA_TYPE PCV description
and issues on standard output the corresponding antenna PCV for a
south oriented antenna.
END_USAGE

die( $USAGE ) if ( @ARGV != 1 );

my $is_rotating     = 0;
my $previous_line   = '';
my %elevation_at;           # hash of elevations according to azimuths
my $number_of_frequency;
my $number_of_frequency_already_met = 0;
open( ANTMOD, $ANTMOD_FILE )
    or die("$!\n");
while ( defined( my $line = <ANTMOD>)){
    chomp $line;
    $line =~ s/\s*$//;  # avoids trouble with unconsistent tailing blanks
    if (!$is_rotating){                         # hasn't found the antenna yet
        if ($line =~ m{\A([\w.+]+)\s+.*                 # first 60 characters
                        TYPE\ /\ SERIAL\ NO\z       # last 20 characters
                        }xm){
            if ($1 eq $ARGV[0]){                    # the one we're looking for?
                $is_rotating    = -1;
                print "$previous_line\n";
                print 'Z'.substr($line,1)."\n";     # new name negins with Z
            }
        }
        $previous_line = $line;
    }
    else {  # $is_rotating
        if ( any { substr($line,60) eq $_ }@STEADY_LINES ){
            print "$line\n";
        }
        elsif (substr($line,60) eq 'END OF ANTENNA'){
            print "$line\n";
            close ANTMOD;
            if ($number_of_frequency_already_met != $number_of_frequency){
                print "warning!"
                        ."waited for $number_of_frequency frequencies\n";
            }
            exit;
        }
        elsif (substr($line,60) eq '# OF FREQUENCIES'){
            $number_of_frequency = unpack( 'A6',$line);
            print "$line\n";
        }
        elsif (substr($line,60) eq 'START OF FREQUENCY'){
            $number_of_frequency_already_met++;
            print "$line\n";
        }
        elsif (substr($line,60) eq 'SINEX CODE'){
            print 'Z'.substr($line,1)."\n";     # new name negins with Z
        }
        elsif (substr($line,60) eq 'NORTH / EAST / UP'){
            my ($north, $east, $up) = unpack('A10A10A10',$line);
            substr($line,0,30)      = sprintf "%10s%10s%10s",map( {sprintf "%.2f",$_} (-$north,-$east,$up));
            print "$line\n";
        }
        elsif ((length($line) == (20*8)) || (length($line) == (18*8))){
            if (substr($line,3,5) eq 'NOAZI'){
                print "$line\n";
            }
            else {
                $elevation_at{sprintf "%.0f",substr($line,0,8)} = substr($line,8);
            }
        }
        elsif (substr($line,60) eq 'END OF FREQUENCY'){
            if ( %elevation_at ){
                for (my $azimuth=0;$azimuth<360;$azimuth+=5){
                    my $old_azimuth = ($azimuth + 180)%360;
                    printf "%8s%s\n",(sprintf '%.1f',$azimuth),$elevation_at{$old_azimuth};
                }
                printf "%8s%s\n",(sprintf '%.1f',360),$elevation_at{180};
            }
            print "$line\n";
        }
        else {
            die(substr($line,61)." is an unknown line!\n");
        }
    }
}
close ANTMOD;
